# -*- coding: utf-8 -*-
# globalPlugins/closingTools.py.
# This file is covered by the GNU General Public License.
# You can read the licence by clicking Help->Licence in the NVDA menu
# or by visiting http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
# Shortcut: pause, Control+Pause, Alt+pause, control+backspace

import globalPluginHandler, addonHandler
import api
import ui
import wx
from keyboardHandler import KeyboardInputGesture
from .closingToolsSettings import ADDON_SUMMARY, ADDON_NAME, announces, ClosingToolsSettingsDialog
import gui
import config
from tones import beep
# We initialize translation support
addonHandler.initTranslation()

def getWindowTitleAndAppModule():
	""" Return the current window title and the corresponding appModule """
	obj=api.getForegroundObject()
	title=obj.name
	# if not isinstance(title,basestring) or not title or title.isspace():
	if not title or title.isspace():
		title=obj.appModule.appName  if obj.appModule else "No title"
	return title, obj.appModule if obj.appModule else "No appModule"

def isClosed (title, appModule):
	newTitle, newAppModule = getWindowTitleAndAppModule()
	if newTitle == title and newAppModule == appModule:
		return False
	return True

class GlobalPlugin(globalPluginHandler.GlobalPlugin):
	# We initialize the scripts category shown on input gestures dialog
	scriptCategory = ADDON_SUMMARY
	# We initialize a timer for repeated scripts.
	timer = None
	# We initialize a variable for reporting value change or not.
	reportValueChange = True
	# We initialize a boulean for avoiding multiple instances when closing all windows.
	closeAllDialog = False

	def __init__(self, *args, **kwargs):
		super (GlobalPlugin, self).__init__(*args, **kwargs)
		# This block ensures compatibility with NVDA versions prior to 2018.2 which includes the settings panel.
		if hasattr (gui, "NVDASettingsDialog"):
			from gui import NVDASettingsDialog
			NVDASettingsDialog.categoryClasses.append(ClosingToolsSettingsDialog)
		else:
			self.createSubMenu ()

	"""def event_foreground(self, obj, nextHandler) :
		isTB = False
		if obj.windowClassName.startswith ("Mozilla") :
			title=str(obj.name)
			#print("titre : " + title)
			if "Thunderbird" in title :
				isTB = True
		if isTB :
			beep(200, 40)
		nextHandler()		 """
	def createSubMenu (self):
		self.toolsMenu = gui.mainFrame.sysTrayIcon.toolsMenu
		closingToolsMenu = wx.Menu ()
		self.mainItem = self.toolsMenu.AppendSubMenu (closingToolsMenu,
		# Translators: Item in the tools menu for the Addon closingTools.
		_("Closing &Tools..."),
		# Translators: The tooltyp text for the closingTools submenu.
		_("{0} add-on and its settings").format (ADDON_NAME))

		addonSettings = closingToolsMenu.Append (wx.ID_ANY,
		# Translators: The name of the item in the closingTools add-on submenu.
		_("{0} add-on se&ttings").format (ADDON_NAME),
		# Translators: The tooltyp text for the item in the closingTools add-on submenu.
		_("Configure the {0} add-on").format (ADDON_NAME))
		gui.mainFrame.sysTrayIcon.Bind (wx.EVT_MENU, self.onAddonSettingsDialog, addonSettings)

	def terminate (self):
		# This block ensures compatibility with NVDA versions prior to 2018.2 which includes the settings panel.
		if hasattr (gui, "NVDASettingsDialog"):
			gui.NVDASettingsDialog.categoryClasses.remove(ClosingToolsSettingsDialog)
		try:
			if wx.version().startswith("4"):
				self.toolsMenu.Remove(self.mainItem)
			else:
				self.toolsMenu.RemoveItem(self.mainItem)
		except:
			pass

	def onAddonSettingsDialog (self, evt):
		gui.mainFrame.prePopup ()
		d = ClosingToolsSettingsDialog (gui.mainFrame)
		d.Show ()
		gui.mainFrame.postPopup ()

	def script_activateClosingToolsSettingsDialog (self, gesture):
		# This block ensures compatibility with NVDA versions prior to 2018.2 which includes the settings panel.
		if hasattr (gui, "NVDASettingsDialog"):
			wx.CallAfter (gui.mainFrame._popupSettingsDialog, gui.NVDASettingsDialog, ClosingToolsSettingsDialog)
		else:
			wx.CallAfter (self.onAddonSettingsDialog, gui.mainFrame)

	# Translators: Message presented in input help mode.
	script_activateClosingToolsSettingsDialog.__doc__ = _("Allows you to open the {0} add-on settings dialog.").format (ADDON_NAME)

	def event_valueChange (self, obj, nextHandler):
		if not self.reportValueChange:
			return
		nextHandler ()

	def initializeTimer (self):
		if self.timer is not None:
			self.timer.Stop()
			self.timer = None

	def closeTabOrWindow (self, title, appModule, wnd = False):
		import time
		msg = ""
		if config.conf["closingTools"]["announceLevel"] == "short":
			msg = announces[2]
		if not wnd: # L'utilisateur souhaite refermer un onglet.
			if config.conf["closingTools"]["announceLevel"] == "long":
				msg = announces[1]
			if appModule.appName == "mp3directcut":
				# Nous définissons le raccourci-clavier à exécuter.
				key = "control+f4"
			else: # Nous ne sommes pas dans mp3DirectCut.wx.CallLater (300, 
				key = "control+w"
			# Nous définissons la commande selon le raccourci-clavier utilisé.
			command = KeyboardInputGesture.fromName (key)
		else: # L'utilisateur souhaite plutôt refermer une fenêtre.
			if config.conf["closingTools"]["announceLevel"] == "long":
				msg = announces[0]
			# On définit notre commande.
			command = KeyboardInputGesture.fromName ("alt+f4")
		# On exécute maintenant notre commande selon chaque contexte.
		self.reportValueChange = False
		command.send()
		time.sleep (0.2)
		if not isClosed (title, appModule):
			self.reportValueChange = True
			return
		# On fait lire notre message de fermeture.		
		ui.message (msg)
		time.sleep (1.0)
		api.processPendingEvents ()
		self.reportValueChange = True

	def script_sendAltF4(self, gesture):
		from scriptHandler import getLastScriptRepeatCount
		title,appModule = getWindowTitleAndAppModule()
		repeat = getLastScriptRepeatCount()
		# On initialise le timer.
		self.initializeTimer ()
		if repeat == 0: # On souhaite refermer la fenêtre.
			self.timer = wx.CallLater (300, self.closeTabOrWindow, title, appModule, True)
		elif repeat == 1: # On souhaite plutôt refermer l'onglet actif.
			self.timer = wx.CallLater (300, self.closeTabOrWindow, title, appModule)
		else: # On souhaite refermer toutes les fenêtres.
			wx.CallLater(50, self.doCloseAllWindows)

	# Translators: message presented when user performes input help for the pose key script
	script_sendAltF4.__doc__ = _(u"Close the focused window by sending Alt+F4")

	def script_sendControlF4(self, gesture):
		KeyboardInputGesture.fromName("control+f4").send()
		title,appModule = getWindowTitleAndAppModule()
		wx.CallLater(100, ui.message, title)
	script_sendControlF4.__doc__ = _(u"Close the  active  tab by sending control+F4")

	def script_sendControlW(self, gesture):
		#ui.message ("Fermer onglet")
		KeyboardInputGesture.fromName("control+w").send()
		title,appModule = getWindowTitleAndAppModule()
		wx.CallLater(100, ui.message, title)
	script_sendControlW.__doc__ = _(u"Close the  active  tab by sending control+w")
	def script_sendWinD(self, gesture):
		wx.CallLater(40, KeyboardInputGesture.fromName("windows+d").send)
	script_sendWinD.__doc__ = _(u"Show the desktop by sending windows+d")
	
	def script_closeAllWindows(self, gesture):
		wx.CallLater(50, self.doCloseAllWindows)
	script_closeAllWindows.__doc__ = _(u"Close all   windows")
	def doCloseAllWindows(self) :
		import os
		fpth = "C:\\Program Files\\Mozilla Thunderbird\\thunderbird.exe"
		os.startfile (fpth)

	
	"""def doCloseAllWindows(self) :
		from gui import messageBox
		import os
		# To avoid multiple instances.
		if self.closeAllDialog:
			return
		self.closeAllDialog = True
		#res = messageBox(_(u"Close all windows ?"), _(u"Close the windows"), wx.YES | wx.NO | wx.ICON_WARNING)
		if res in (wx.YES, wx.NO):
			self.closeAllDialog = False
			if res == wx.YES:
				#fpth = os.path.join (addonHandler.getCodeAddon().path, "utils", "CloseAllWindows.exe")
				fpth = "C:\\Program Files\\Mozilla Thunderbird\\thunderbird.exe"
				if not os.path.exists(fpth) :
					messageBox(_(u"File not found : ") + fpth, "Close all windows", wx.CLOSE|wx.ICON_WARNING)
					self.closeAllDialog = False
					return
				os.startfile (fpth) """

	__gestures={
		"kb:pause": "sendAltF4",
		#"kb:control+backspace": "sendControlF4",
		"kb:control+break": "sendControlW",
		"kb:alt+pause": "closeAllWindows",
}
