# -*- coding: utf-8 -*-
from __future__ import unicode_literals # To ensure coding compatibility with python 2 and 3.
# globalPlugins/closingToolsSettings.py.
# This file is covered by the GNU General Public License.
# You can read the licence by clicking Help->Licence in the NVDA menu
# or by visiting http://www.gnu.org/licenses/old-l+backspace

import wx
import gui
import config

# We initialize translation support
import addonHandler
addonHandler.initTranslation ()

# This block ensures compatibility with NVDA versions prior to 2018.2 which includes the settings panel.
try:
	from gui import SettingsPanel as SettingsDialog
except ImportError:
	from gui.settingsDialogs import SettingsDialog

### Constants
ADDON_SUMMARY = addonHandler.getCodeAddon ().manifest["summary"]
ADDON_NAME = addonHandler.getCodeAddon ().manifest["name"]

announces = (
	# Translators: The long announce when closing the window.
	_("Closing the Window"),
	# Translators: The long announce when closing the tab.
	_("Closing the tab"),
	# Translators: The short announce when closing the window or tab.
	_("Closing")
	)

confSpec = {
	"announceLevel" : "string(default = long)",
	}
config.conf.spec["closingTools"] = confSpec

class ClosingToolsSettingsDialog (SettingsDialog):

	# Translators: The title of the add-on configuration dialog box.
	title = _("Configuration of the addon {0}").format (ADDON_NAME) if not hasattr (gui, "NVDASettingsDialog") else _("Closing tools")
	ANNOUNCE_LEVELS = (
		("short",
		# Translators: Level for short announces.
		_("Short")),
		("long",
		# Translators: Level for long announces.
		_("Long")),
		("off",
		# Translators: Level to disable announces.
		_("Off"))
	)

	def makeSettings (self, settingsSizer):
		# Translators: The label for an item to select the level of the announces (short, long or disabled).
		self.announceLevelText = _("Level of the &announces:")

		# This try/except/else block has been added to ensure the compatibility of the add-on with the NVDA versions that preceded version 2016.4, which included the gui.guiHelper module.
		try:
			import gui.guiHelper
		except ImportError:
			self.showSettingsDialog (settingsSizer = settingsSizer)
		else:
			self.showSettingsDialogForGuiHelper (settingsSizer = settingsSizer)

	def showSettingsDialogForGuiHelper (self, settingsSizer):
		settingsSizerHelper = gui.guiHelper.BoxSizerHelper (self, sizer = settingsSizer)
		announceLevelChoices = [name for level, name in self.ANNOUNCE_LEVELS]
		self.announceLevelsList = settingsSizerHelper.addLabeledControl(self.announceLevelText, wx.Choice, choices = announceLevelChoices)
		curLevel = config.conf["closingTools"]["announceLevel"]
		for index, (level, name) in enumerate(self.ANNOUNCE_LEVELS):
			if level == curLevel:
				self.announceLevelsList.SetSelection(index)
				break

	def showSettingsDialog (self, settingsSizer):
		dialogSizer = wx.BoxSizer (wx.VERTICAL)
		announceLevelChoices = [name for level, name in self.ANNOUNCE_LEVELS]
		announce = wx.StaticText (self,label=self.announceLevelText)
		dialogSizer.Add (announce)
		self.announceLevelsList = wx.Choice (self, choices = announceLevelChoices)
		curLevel = config.conf["closingTools"]["announceLevel"]
		for index, (level, name) in enumerate(self.ANNOUNCE_LEVELS):
			if level == curLevel:
				self.announceLevelsList.SetSelection(index)
				break
		dialogSizer.Add (self.announceLevelsList)
		self.announceLevelsList.Enabled = self.enableAnnouncesCheckBox.IsChecked ()
		settingsSizer.Add (dialogSizer, border=10, flag=wx.BOTTOM)

	def postInit (self):
		self.announceLevelsList.SetFocus ()

	def onOk (self, evt):
		announceLevel = self.ANNOUNCE_LEVELS[self.announceLevelsList.GetSelection()][0]
		config.conf["closingTools"]["announceLevel"] = announceLevel
		super (ClosingToolsSettingsDialog, self).onOk (evt)

	def onSave (self):
		announceLevel = self.ANNOUNCE_LEVELS[self.announceLevelsList.GetSelection()][0]
		config.conf["closingTools"]["announceLevel"] = announceLevel
